import java.io.BufferedReader;
import java.io.IOException;
import java.io.Writer;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Collections;

public class Daten {
  private ArrayList<Datenpunkt> dieDatenpunkte= new ArrayList<Datenpunkt>();
  public ArrayList<Integer> labels;// = new ArrayList<String>();
  private double xMin;
  private double xMax;
  private double yMin;
  private double yMax;
  private double xWerte;   // Wertebereich
  private double yWerte;
  public int distanzFunktion =0; // Euklid Manhatten Maximum

  public void add(double x, double y, int label) {
    dieDatenpunkte.add(new Datenpunkt(x, y, label));
  }
  public int getAnzahl() {
    return dieDatenpunkte.size();
  }
  public void sortiere() {
    Collections.sort(dieDatenpunkte);
  }
  public Datenpunkt getDatenpunkt(int n) {
    return dieDatenpunkte.get(n);
  }

  public double getxMin(){return xMin;}
  public double getxMax(){return xMax;}
  public double getyMin(){return yMin;}
  public double getyMax(){return yMax;}
  public double getxWerte(){return xWerte;}
  public double getyWerte(){return yWerte;}

  public String analysiereDaten(){ // Wertebereich ermitteln
    String rueckgabe;
    Datenpunkt dp;
    //Daten da = dieSteuerung.getDaten();
    xMax=xMin=dieDatenpunkte.get(0).getX();
    yMax=yMin=dieDatenpunkte.get(0).getY();
    for (int i = 1; i< dieDatenpunkte.size();i++){
      dp=dieDatenpunkte.get(i);
      if(dp.getX()>xMax)xMax=dp.getX();
      if(dp.getX()<xMin)xMin=dp.getX();
      if(dp.getY()>yMax)yMax=dp.getY();
      if(dp.getY()<yMin)yMin=dp.getY();
    }
    rueckgabe=String.format("#Datensätze: %d xmin: %.2f xmax %.2f ymin: %.2f ymax: %.2f",dieDatenpunkte.size(),
        xMin,xMax,yMin,yMax);
    xWerte=xMax-xMin;
    yWerte=yMax-yMin;
    return rueckgabe;
  }
  public void berechneDistanzen(double x, double y) { // ermittelt für alle Datenpunkte Distanzen zu x,y
    for (int i = 0; i < dieDatenpunkte.size(); i++){
      switch (distanzFunktion){
        case 0: // Euklid
          dieDatenpunkte.get(i).setDistanz(Math.sqrt(Math.pow(x - dieDatenpunkte.get(i).getX(), 2.0) + Math.pow(y - dieDatenpunkte.get(i).getY(), 2.0)));
          break;
        case 1: // Manhatten
          dieDatenpunkte.get(i).setDistanz(Math.abs(x-dieDatenpunkte.get(i).getX())+Math.abs(y-dieDatenpunkte.get(i).getY()));
          break;
        case 2: // Maximum
          dieDatenpunkte.get(i).setDistanz(Math.max(Math.abs(x-dieDatenpunkte.get(i).getX()),Math.abs(y-dieDatenpunkte.get(i).getY())));
          break;
      }
    }
  }
  public double ermittleDistanz(Datenpunkt d1,Datenpunkt d2){ // ermittelt die Entfernung zu d
    switch (distanzFunktion){
      case 0: // Euklid
        return Math.sqrt(Math.pow(d1.getX()- d2.getX(), 2.0) + Math.pow(d1.getY()- d2.getY(), 2.0));
        //break;
      case 1: // Manhatten
        return Math.abs(d1.getX() - d2.getX())+Math.abs(d1.getY() - d2.getY());
        //break;
      case 2: // Maximum
        return Math.max(Math.abs(d1.getX() - d2.getX()),Math.abs(d1.getY() - d2.getY()));
        //break;
    }
    return 0;
  }
  public int majorLabel(int k){ // Label (Index) des häufgisten Labels in der Nachbarschaft
    if (k==1) return getDatenpunkt(0).getLabel();
    if (k==2){ // mindestens 2 Datenpunkte nötig
      if(getDatenpunkt(0).getLabel()!=getDatenpunkt(1).getLabel()) return -1; // unentschieden
      return getDatenpunkt(0).getLabel();
    }
    int[] vorkommen = new int[9]; //dieDaten.labels.size() noch umschreiben nach labelIndex
    for(int i=0;i<k;i++){ // Vorkommen pro Label zählen
      vorkommen[getDatenpunkt(i).getLabel()]++;
    }
    int m1=0,m2=0; // index der häufigsten Labels
    for(int i=0;i<vorkommen.length;i++){ // für alle Labels
      if(vorkommen[i]>=vorkommen[m1]){
        m2=m1;
        m1=i;
      }
    }
    if(vorkommen[m1]==vorkommen[m2]) return -1; // unentschieden
    return m1; // Label mit dem häufgisten Vorkommen
  }
  public void leseCSV(Path pfad){
    int i;
    String zeile;
    String[] attribute;
    try { // versuchen wir mal folgenden Code aus zu fuehren
      BufferedReader ein = Files.newBufferedReader(pfad); // oeffne Datenstrom
      while ((zeile = ein.readLine())!= null){ // solange Datensaetze vorhanden, lese Zeile
        zeile=zeile.replace(',','.'); // damit Doubleparsen geht
        attribute = zeile.split(";");
        add(Double.parseDouble(attribute[0]), Double.parseDouble(attribute[1]), Integer.parseInt(attribute[2]));
      }
      ein.close(); // schliesse Datenstrom
    } catch (IOException e) { // falls ein IO-Ausnahmefehler auftritt tue folgendes
      System.out.println("IOException: " + e); // gib den Fehler auf der Konsole aus
    }
    labels = new ArrayList<Integer>();
    for (Datenpunkt dp:dieDatenpunkte){
      if(!labels.contains(dp.getLabel()))labels.add(dp.getLabel());
    }
  }
  public void schreibeCSV(Path pfad){
    int i;
    try {  // versuchen wir mal folgenden Code aus zu fuehren
      Writer aus = Files.newBufferedWriter(pfad); //Datenschreiber
      for(i=0;i<getAnzahl();i++){
        aus.write(getDatenpunkt(i).toCSV()+"\n"); //schreibe
      }
      aus.close(); // schliesse Datenstrom
    } catch (IOException e) {  //falls ein IO-Ausnahmefehler
      System.out.println("IOException: " + e);  //Fehler ausgeben
    }
  }
}