import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

/**
 * Title:        k_App
 * Description:  Visualisierung des K-Nächste Nachbarn Verfahrens
 * Copyright:    Copyright (c) 2024
 * Company:      MezMedia
 * @author Oliver Mezger
 * @version 0.8
 */

public class Oberflaeche extends JFrame { // Das ist der Fensterrahmen des Programms
  public static final String version = "k-NN+Means V0.8 (c) Oliver Mezger 3.3.2024 CC BY-SA 4.0";
  public Steuerung dieSteuerung;
  public  NordPanel dasNordPanel = new NordPanel(this);
  public  MittePanel dasMittePanel = new MittePanel(this);
  public  OstPanel dasOstPanel = new OstPanel(this);
  public  SuedPanel dasSuedPanel = new SuedPanel();
  public enum AnzeigenModusTyp {KLICKEN,MAUS_FOLGEN,PUNKT_EINFUEGEN,K_MEANS};
  public AnzeigenModusTyp anzeigenModus = AnzeigenModusTyp.KLICKEN;
  public static int datenPunktSize = 7; // Grösse der DatenPunkte
  public Oberflaeche() {    // Die Steuerung ist eine extra Klasse
    getContentPane().add(new MyContainer(this)); // Den Container in den Fensterrahmen eingebaut
    setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE ); // Beenden wenn Rahmen geschlossen
    dieSteuerung = new Steuerung(this);
  }
  public void ausgebenDateiName(String dn){
    dasNordPanel.dateinameLabel.setText("Datenquelle: "+dn);
  }
  public void ausgebenDatenInfo(String di){
    dasSuedPanel.datenInfoLabel.setText(di);
  }

  public static void main(String[] args) { // Main Methode zum starten der Applikation
    Oberflaeche dieOberflaeche = new Oberflaeche(); // Das Fenster wird erzeugt
    dieOberflaeche.setSize(850,650);    // Die Groesse wird festgelegt
    dieOberflaeche.setTitle(version);
    dieOberflaeche.setVisible(true);
  }
  public void anzeigenModusNeu(int i){ // Oberfläche für neuen Anzeigenmodus einstellen
    dasOstPanel.datenKlasseLabel.setVisible(false);
    dasOstPanel.datenKlasseSlider.setVisible(false);
    dasOstPanel.kMeansZentrenButton.setVisible(false);
    dasOstPanel.kMeansLabelnButton.setVisible(false);
    dasOstPanel.kMeansSchwerpunkteButton.setVisible(false);
    switch (i){
      case 0: // nur Klicken
        anzeigenModus=AnzeigenModusTyp.KLICKEN;
        dieSteuerung.mausPunkt =false;
        break;
      case 1: // Maus Folgen
        anzeigenModus=AnzeigenModusTyp.MAUS_FOLGEN;
        break;
      case 2: // Daten hinzufügen
        anzeigenModus=AnzeigenModusTyp.PUNKT_EINFUEGEN;
        dieSteuerung.mausPunkt =false;
        dasOstPanel.datenKlasseLabel.setVisible(true);
        dasOstPanel.datenKlasseSlider.setVisible(true);
        break;
      case 3: // k-Means
        anzeigenModus=AnzeigenModusTyp.K_MEANS;
        dieSteuerung.mausPunkt =false;
        dasOstPanel.kMeansZentrenButton.setVisible(true);
        dasOstPanel.kMeansLabelnButton.setVisible(true);
        dasOstPanel.kMeansSchwerpunkteButton.setVisible(true);
        break;
    }
    dasMittePanel.repaint();
  }
}
class MyContainer extends Container { // Ein Container nimmt die einzelnen Panele auf
  public MyContainer(Oberflaeche o){
    setLayout(new BorderLayout(3,3));
    add(o.dasNordPanel, BorderLayout.NORTH );
    add(o.dasOstPanel, BorderLayout.EAST );
    add(o.dasMittePanel, BorderLayout.CENTER );
    add(o.dasSuedPanel, BorderLayout.SOUTH );
  } 
}
class NordPanel extends JPanel{
  private final Oberflaeche dieOberflaeche;
  public final JLabel dateinameLabel = new JLabel("");
  //https://docs.oracle.com/javase/tutorial/uiswing/components/filechooser.html
  final JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));
  public final String distanzFunktionen[] = {"Euklid","Manhatten","Maximum"};
  public  JList distanzFunktionJList = new JList(distanzFunktionen);
  public final String anzeigeModi[] = {"Ins Bild Klicken","Maus folgen","DatenP hinzu","K-Means"};
  public JList anzeigenModusJList = new JList(anzeigeModi);
  public NordPanel(Oberflaeche o){
    dieOberflaeche=o;
    setBackground(Color.decode("0xFFFF66"));
    setLayout(new FlowLayout());
    add(dateinameLabel);
    JButton tmp;
    tmp = new JButton("Neue Datei öffnen");
    add (tmp);
    fc.addChoosableFileFilter(new CSVFilter()); // damit nur CSV angezeigt werden
    fc.setAcceptAllFileFilterUsed(false);
    //fc.setCurrentDirectory();
    tmp.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        int returnVal = fc.showOpenDialog(dieOberflaeche); // nahe des Fensters anzeigen
        if (returnVal == JFileChooser.APPROVE_OPTION) {
          File file = fc.getSelectedFile();
          dieOberflaeche.dieSteuerung.ladeDaten(file.toPath());
          dieOberflaeche.dieSteuerung.mausPunkt =false;
          dieOberflaeche.dasMittePanel.repaint();
        }
      }
    });
    tmp = new JButton("speichern");
    add (tmp);

    tmp.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        int returnVal = fc.showSaveDialog(dieOberflaeche); // nahe des Fensters anzeigen
        if (returnVal == JFileChooser.APPROVE_OPTION) {
          File file = fc.getSelectedFile();
          dieOberflaeche.dieSteuerung.speichereDaten(file.toPath());
        }
      }
    });
    add(distanzFunktionJList);
    distanzFunktionJList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    distanzFunktionJList.setSelectedIndex(0);
    distanzFunktionJList.addListSelectionListener(new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
        dieOberflaeche.dieSteuerung.getDaten().distanzFunktion=distanzFunktionJList.getSelectedIndex();
        dieOberflaeche.dieSteuerung.bestimmeNachbarn();
        dieOberflaeche.dasMittePanel.repaint();
      }
    });
    add(anzeigenModusJList);
    anzeigenModusJList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    anzeigenModusJList.setSelectedIndex(0);
    anzeigenModusJList.addListSelectionListener(new ListSelectionListener() {
      @Override
      public void valueChanged(ListSelectionEvent e) {
        dieOberflaeche.anzeigenModusNeu(anzeigenModusJList.getSelectedIndex());
      }
    });
  }
}
class CSVFilter extends FileFilter{ // nur CSV sind gewünscht
  public boolean accept(File f) {
    if (f.isDirectory()) { // Unterverzeichnisse willkommen
      return true;
    }
    String filename = f.toString();
    int index = filename.lastIndexOf('.');
    if(index>0){
      String extension = filename.substring(index+1);
      if (extension != null) {
        return extension.equals("csv");
      }
    }
    return false;
  }
  public String getDescription() {
    return "Nur CSV";
  }
}
class OstPanel extends JPanel{
  private final Oberflaeche dieOberflaeche;
  public final JLabel kWertLabel = new JLabel("K-Wert: 3");
  public final JLabel dpSizeLabel = new JLabel("Datenpunktgrösse: "+Oberflaeche.datenPunktSize);
  public final JLabel nachbarnLabel = new JLabel("Nachbarn");
  public final JLabel mausKoordinatenLabel = new JLabel("Mauskoordinaten");
  public final JLabel datenKlasseLabel = new JLabel("Label: 1");
  public final JSlider datenKlasseSlider = new JSlider();
  public final JButton kMeansZentrenButton =  new JButton("Zentren setzen");
  public final JButton kMeansLabelnButton =  new JButton("K-Means Labeln");

  public final JButton kMeansSchwerpunkteButton =  new JButton("Schwerpunkte neu");

  public OstPanel(Oberflaeche o){
    dieOberflaeche = o;
    setBackground(Color.decode("0xccffcc"));
    setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
    add(Box.createVerticalStrut(20));
    add(kWertLabel);
    JSlider kWerteSlider = new JSlider();
    kWerteSlider.setMinimum(1);
    kWerteSlider.setMaximum(8);
    kWerteSlider.setValue(3);
    kWerteSlider.setSize(30,20); // zeigt keine Wirkung
    kWerteSlider.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent e) {
        kWertLabel.setText("K-Wert: "+kWerteSlider.getValue());
        dieOberflaeche.dieSteuerung.setK(kWerteSlider.getValue());
        dieOberflaeche.dieSteuerung.anzeigenLabels();
        dieOberflaeche.dasMittePanel.repaint();
      }
    });
    add(kWerteSlider);
    add(Box.createVerticalStrut(20));
    add(dpSizeLabel);
    JSlider punktSize = new JSlider();
    punktSize.setMinimum(4);
    punktSize.setMaximum(20);
    punktSize.setValue(Oberflaeche.datenPunktSize);
    //punktSize.setSize(30,20); // zeigt keine Wirkung
    punktSize.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent e) {
        dieOberflaeche.datenPunktSize=punktSize.getValue();
        dpSizeLabel.setText("Datenpunktgrösse: "+punktSize.getValue());
        dieOberflaeche.dasMittePanel.repaint();
      }
    });
    add(punktSize);
    add(Box.createVerticalStrut(20));
    nachbarnLabel.setBorder(BorderFactory.createRaisedBevelBorder());
    nachbarnLabel.setOpaque(true);
    add(nachbarnLabel);
    add(Box.createVerticalStrut(20));
    add(mausKoordinatenLabel);
    add(Box.createVerticalStrut(20));
    datenKlasseLabel.setVisible(false);
    datenKlasseLabel.setBorder(BorderFactory.createRaisedBevelBorder());
    datenKlasseLabel.setOpaque(true);
    datenKlasseLabel.setBackground(Color.RED);
    add(datenKlasseLabel);
    add(Box.createVerticalStrut(20));
    datenKlasseSlider.setMinimum(1);
    datenKlasseSlider.setMaximum(9);
    datenKlasseSlider.setValue(1);
    datenKlasseSlider.setVisible(false);
    datenKlasseSlider.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent e) {
        datenKlasseLabel.setText("Label: "+datenKlasseSlider.getValue());
        dieOberflaeche.dasOstPanel.datenKlasseLabel.setBackground(dieOberflaeche.dieSteuerung.labelFarben[datenKlasseSlider.getValue()]);
      }
    });
    add(datenKlasseSlider);
    kMeansZentrenButton.setVisible(false);
    kMeansZentrenButton.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        dieOberflaeche.dieSteuerung.bestimmeZentren();
        dieOberflaeche.dasMittePanel.repaint();
      }
    });
    add(kMeansZentrenButton);
    kMeansLabelnButton.setVisible(false);
    kMeansLabelnButton.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        mausKoordinatenLabel.setText(String.format("∑ Abstandsquadrate: %5.1f",dieOberflaeche.dieSteuerung.kMeansLabeln()));
        dieOberflaeche.dasMittePanel.repaint();
      }
    });
    add(kMeansLabelnButton);
    kMeansSchwerpunkteButton.setVisible(false);
    kMeansSchwerpunkteButton.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        if(dieOberflaeche.dieSteuerung.kMeansSchwerpunkte()){
          mausKoordinatenLabel.setText("Zentren sind neu");
        }
        else {
          mausKoordinatenLabel.setText("Zentren unverändert");
        }
        dieOberflaeche.dasMittePanel.repaint();
      }
    });
    add(kMeansSchwerpunkteButton);
  }
}
class MittePanel extends JPanel implements MouseMotionListener,ComponentListener{
  private final Oberflaeche dieOberflaeche;
  public MittePanel(Oberflaeche o){
    dieOberflaeche=o;
    setBackground(Color.white);
    addMouseMotionListener(this);
    addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent e) {
        double x = dieOberflaeche.dieSteuerung.dasSchaubild.xPixelToWert(e.getX());
        double y = dieOberflaeche.dieSteuerung.dasSchaubild.yPixelToWert(e.getY());
        switch (dieOberflaeche.anzeigenModus){
          case PUNKT_EINFUEGEN -> dieOberflaeche.dieSteuerung.getDaten().add(x,y,dieOberflaeche.dasOstPanel.datenKlasseSlider.getValue());
          default -> dieOberflaeche.dieSteuerung.setMausposition(x,y);
        }
        repaint();
      }
    });
    addComponentListener(this);
  }
  public void mouseMoved(MouseEvent e) {
    double x = dieOberflaeche.dieSteuerung.dasSchaubild.xPixelToWert(e.getX());
    double y = dieOberflaeche.dieSteuerung.dasSchaubild.yPixelToWert(e.getY());
    dieOberflaeche.dasOstPanel.mausKoordinatenLabel.setText(String.format("Maus x:%5.1f y:%5.1f",x,y));
    switch (dieOberflaeche.anzeigenModus){
      case MAUS_FOLGEN:
        dieOberflaeche.dieSteuerung.setMausposition(x,y);
        repaint();
    }
  }
  public void mouseDragged(MouseEvent e){}
  public void paint(Graphics g) {
    super.paint(g);
    dieOberflaeche.dieSteuerung.maleSchaubild(g,Oberflaeche.datenPunktSize);
  }
  public void componentResized(ComponentEvent ce){
    dieOberflaeche.dieSteuerung.setPanelBreite(getWidth(),getHeight());
  }
  public void componentHidden(ComponentEvent ce) {}
  public void componentShown(ComponentEvent ce) {}
  public void componentMoved(ComponentEvent ce) {}
}
class SuedPanel extends JPanel{
  public final JLabel datenInfoLabel = new JLabel("DatenInfo");
  public SuedPanel(){
    setBackground(Color.orange);
    setLayout(new FlowLayout());
    add(datenInfoLabel);
  }
}