import java.awt.*;

public class Schaubild {
  private static int nMargin = 20;
  private static int oMargin = 20;
  private static int sMargin = 50;
  private static int wMargin = 50;
  private Steuerung dieSteuerung; // wird benötigt um an die Daten zu kommen
  private int xPixel;
  private int yPixel;
  private double xMin;
  private double xMax;
  private double yMin;
  private double yMax;
  private double xWerte;   // Wertebereich
  private double yWerte;
  private double xFaktor;  // Umrechnungsfaktor Wert->Pixel
  private double yFaktor;
  private double xTeilung; // Teilstrichabstand
  private double yTeilung;
  public Schaubild(Steuerung s){
    dieSteuerung = s;
  }
  public String analysiereDaten(){ // Wertebereich ermitteln
    String rueckgabe;
    Daten da = dieSteuerung.getDaten();
    rueckgabe=da.analysiereDaten(); // schau dir die Daten an
    double randX = (da.getxWerte())*0.1; // 10% Rand
    double randY = (da.getyWerte())*0.1; // 10% Rand
    xMin=Math.round(da.getxMin()-randX); // Schaubildbereich anpassen
    xMax=Math.round(da.getxMax()+randX);
    yMin=Math.round(da.getyMin()-randY);
    yMax=Math.round(da.getyMax()+randY);
    return rueckgabe;
  }
  public void setPanelBreite(int x,int y){ // wenn sich Mittepanel verändert
    xPixel=x-oMargin-wMargin;
    yPixel=y-nMargin-sMargin;
    xWerte=xMax-xMin;
    yWerte=yMax-yMin;
    xFaktor=xPixel/xWerte;
    yFaktor=yPixel/yWerte;
    xTeilung=optTeilung(xWerte/10);
    yTeilung=optTeilung(yWerte/10);
    //System.out.format("xPixel %d yPixel: %d xWerte %.2f xFaktor %.2f\n",xPixel,yPixel,xWerte,xFaktor);
  }
  private double optTeilung(double t){ // ToDo schönere Teilungen 0.1 0.2 0.5 1.0 1.5 2 5
    if(t<1.0) return 0.5;
    if(t<1.5) return 1.0;
    if(t<2.0) return 1.5;
    if(t<5.0) return 2.0;
    if(t<10.0) return 5.0;
    return 10.0;
  }
  public int xWertToPixel(double w){
    return (int) Math.round((w - xMin) * xFaktor) + wMargin;
  }
  public double xPixelToWert(int x){
    return (x-wMargin)/xFaktor+xMin;
  }
  public int yWertToPixel(double w){
    return nMargin+yPixel-(int) Math.round((w - yMin) * yFaktor);
  }
  public double yPixelToWert(int y){
    return (nMargin+yPixel-y)/yFaktor+yMin;
  }
  public void maleAchsen(Graphics g){
    // male Gitter
    g.setColor(Color.LIGHT_GRAY);
    for (double i = xMin; i <= xMax; i = i + xTeilung) { // vertikal
      g.drawLine(xWertToPixel(i), nMargin, xWertToPixel(i), nMargin+yPixel);
    }
    for (double i = yMin; i <= yMax; i = i + yTeilung) { // horizontal
      g.drawLine(wMargin, yWertToPixel(i), wMargin+xPixel, yWertToPixel(i));
    }
    g.setColor(Color.BLACK);
    g.drawLine(wMargin, nMargin, wMargin, nMargin+yPixel);   //y-Achse
    g.drawLine(wMargin, nMargin+yPixel, wMargin+xPixel, nMargin+yPixel); //x-Achse
    // male Achsenstriche und Werte
    for (double i = xMin; i <= xMax; i = i + xTeilung) {
      g.drawLine(xWertToPixel(i), nMargin+yPixel, xWertToPixel(i), nMargin+yPixel+8);// Strich x-Achse
      if(xTeilung<2.0)
        g.drawString(String.format("%3.1f",i), xWertToPixel(i)- 10, nMargin+yPixel+25); // x-Werte
      else
        g.drawString(String.format("%3.0f",i), xWertToPixel(i)- 10, nMargin+yPixel+25); // x-Werte
    }
    for (double i = yMin; i <= yMax; i = i + yTeilung) {
      g.drawLine(wMargin-8, yWertToPixel(i), wMargin, yWertToPixel(i));             //Strich y-Achse
      if(yTeilung<2.0)
        g.drawString(String.format("%3.1f",i), wMargin-35, yWertToPixel(i)+5); // y-Werte
      else
        g.drawString(String.format("%3.0f",i), wMargin-35, yWertToPixel(i)+5); // y-Werte
    }
  }
  public void maleDaten(Graphics g, int dps){ // Datenpunkte mit Durchmesser dps malen
    Daten d = dieSteuerung.getDaten();
    for (int i = 0; i < d.getAnzahl(); i++) {
      if(d.getDatenpunkt(i).getLabel()<dieSteuerung.labelFarben.length)
        g.setColor(dieSteuerung.labelFarben[d.getDatenpunkt(i).getLabel()]);
      else g.setColor(Color.BLACK);
      g.fillOval(xWertToPixel(d.getDatenpunkt(i).getX())-dps/2,yWertToPixel(d.getDatenpunkt(i).getY())-dps/2,dps,dps);
      g.setColor(Color.BLACK);
      g.drawOval(xWertToPixel(d.getDatenpunkt(i).getX())-dps/2,yWertToPixel(d.getDatenpunkt(i).getY())-dps/2,dps,dps);
    }
  }
  public void maleZentren(Graphics g, int dps){ // Datenpunkte mit Durchmesser dps malen
    Daten d = dieSteuerung.getZentren();
    if(d==null)return;
    g.setFont(new Font("Arial",Font.BOLD,dps*3));
    for (int i = 0; i < d.getAnzahl(); i++) {
      if(d.getDatenpunkt(i).getLabel()<dieSteuerung.labelFarben.length)
        g.setColor(dieSteuerung.labelFarben[d.getDatenpunkt(i).getLabel()]);
      else g.setColor(Color.BLACK);
      g.drawString("X",xWertToPixel(d.getDatenpunkt(i).getX())-dps/2,yWertToPixel(d.getDatenpunkt(i).getY())-dps/2);
    }
    // g.setFont(new Font("Arial",Font.BOLD,dps*2));
    // g.drawString("😀1️⃣⛳️💕x🔻♢✦♦︎▲✘★◼︎",200,200);
  }
  public void maleMausVerbindungen(Graphics g, double x, double y){
    Daten d = dieSteuerung.getDaten();
    g.setColor(Color.BLACK);
    for (int i = 0; i < dieSteuerung.k; i++) { // für die k nächsten Nachbarn
      g.drawLine(xWertToPixel(d.getDatenpunkt(i).getX()),yWertToPixel(d.getDatenpunkt(i).getY()),xWertToPixel(x),yWertToPixel(y)); // male Verbindung zu Mausposition
    }
  }
}