import java.awt.*;
import java.nio.file.Path;
import java.nio.file.Paths;

public class Steuerung {
  private Oberflaeche dieOberflaeche;
  private Daten dieDaten;
  private Daten dieZentren; // für K-Means
  public Schaubild dasSchaubild;
  public int k=3;
  public double mausX;
  public double mausY;
  public boolean mausPunkt = false; // ist ein MausPunkt gesetzt?
  public static Path pfad = Paths.get("daten.csv"); // Dateipfad
  public static Color labelFarben[]={Color.WHITE,Color.RED,Color.decode("0xeecc33"),Color.decode("0xAAee33"),Color.decode("0xAA66aa"),
      Color.BLUE,Color.ORANGE,Color.MAGENTA,Color.PINK,Color.YELLOW,Color.BLACK};
  public Steuerung(Oberflaeche o) {
    dieOberflaeche = o;
    dasSchaubild = new Schaubild(this);
    ladeDaten(pfad);
  }
  public void setK(int k){
    this.k=k;
  }
  public void addDatenpunkt(int l){
    dieDaten.add(mausX, mausY, l);
  }
  public void setPanelBreite(int x,int y){
    dasSchaubild.setPanelBreite(x,y);
  }
  public Daten getDaten(){ // damit Schaubild an Daten kommt
    return dieDaten;
  }
  public Daten getZentren(){return dieZentren;}
  public void ladeDaten(Path pfad){
    dieDaten = new Daten();
    dieZentren = null;
    dieDaten.leseCSV(pfad);
    dieOberflaeche.ausgebenDatenInfo(dasSchaubild.analysiereDaten());
    dieOberflaeche.ausgebenDateiName(pfad.getFileName().toString());
    dasSchaubild.setPanelBreite(dieOberflaeche.dasMittePanel.getWidth(),dieOberflaeche.dasMittePanel.getHeight());
  }
  public void speichereDaten(Path pfad){
    dieDaten.schreibeCSV(pfad);
    dieOberflaeche.ausgebenDateiName(pfad.getFileName().toString());
  }
  public void setMausposition(double x,double y){
    mausX=x;
    mausY=y;
    mausPunkt =true;
    switch (dieOberflaeche.anzeigenModus){
      case MAUS_FOLGEN:
      case KLICKEN:
        bestimmeNachbarn();
        break;
    }
  }
  public void bestimmeNachbarn(){
    dieDaten.berechneDistanzen(mausX,mausY);
    dieDaten.sortiere();
    anzeigenLabels();
  }
  public void maleSchaubild(Graphics g,int dps){
    dasSchaubild.maleAchsen(g);
    dasSchaubild.maleDaten(g,dps);
    if(mausPunkt){
      switch (dieOberflaeche.anzeigenModus){
        case KLICKEN:
        case MAUS_FOLGEN:
          dasSchaubild.maleMausVerbindungen(g,mausX,mausY);
      }
    }
    if(dieOberflaeche.anzeigenModus== Oberflaeche.AnzeigenModusTyp.K_MEANS){
      dasSchaubild.maleZentren(g,dps);
    }
  }
  public void anzeigenLabels(){
    int mj = dieDaten.majorLabel(k);
    if (mj<0){ // keine Lösung
      dieOberflaeche.dasOstPanel.nachbarnLabel.setText("Keine Lösung");
      dieOberflaeche.dasOstPanel.nachbarnLabel.setBackground(Color.WHITE);
    }
    else{
      if(mj<labelFarben.length) {
        dieOberflaeche.dasOstPanel.nachbarnLabel.setBackground(labelFarben[mj]);
      }
      else{
        dieOberflaeche.dasOstPanel.nachbarnLabel.setBackground(Color.WHITE);
      }
      dieOberflaeche.dasOstPanel.nachbarnLabel.setText("Label: " + mj);
    }
  }
  public void bestimmeZentren(){
    dieZentren = new Daten();
    double x,y;
    for (int i=1;i<=k;i++){ // i gibt Labelfarbe vor
      x = dieDaten.getxMin()+Math.random()*dieDaten.getxWerte();
      y = dieDaten.getyMin()+Math.random()*dieDaten.getyWerte();
      dieZentren.add(x,y,i); // neues Zentrum einfügen
    }
  }
  public double kMeansLabeln(){ // Label setzen Rückgabe Summe der quadrieren Abstände zum nächsten Zentrum
    double abstaende=0;
    if(dieZentren==null)return 0;
    double minDist,dist;
    int nz; // nächstes Zentrum
    for (int i=0;i<dieDaten.getAnzahl();i++){
      nz=0;
      minDist = dieDaten.ermittleDistanz(dieDaten.getDatenpunkt(i),dieZentren.getDatenpunkt(0));
      for (int z=1;z<dieZentren.getAnzahl();z++){
        dist = dieDaten.ermittleDistanz(dieDaten.getDatenpunkt(i),dieZentren.getDatenpunkt(z));
        if (dist<minDist){
          minDist=dist;
          nz=z;
        }
      }
      dieDaten.getDatenpunkt(i).setLabel(dieZentren.getDatenpunkt(nz).getLabel());
      abstaende+= minDist*minDist; // quadrierten Abstand aufaddieren
    }
    return abstaende;
  }
  public boolean kMeansSchwerpunkte(){ // Schwerpunkte neu setzen, Rückgabe wurde Zentrum verlagert
    boolean neuZentrum=false; // wurde ein Zentrum verschoben?
    if(dieZentren==null)return false;
    double xsum,ysum;
    int anz;
    Datenpunkt dp,zp;
    for(int z=0;z<dieZentren.getAnzahl();z++){ // für alle Zentren
      xsum=0;
      ysum=0;
      anz=0;
      zp=dieZentren.getDatenpunkt(z);
      int label = zp.getLabel();
      for (int i=0;i<dieDaten.getAnzahl();i++){ // alle Datenpunkte durchgehen
        dp=dieDaten.getDatenpunkt(i);
        if(dp.getLabel()==label){ // wenn Datenpunkt passt
          xsum+=dp.getX();
          ysum+=dp.getY();
          anz++;
        }
      }
      xsum/=anz;
      ysum/=anz;
      if(zp.getX()!=xsum || zp.getY()!= ysum){ // hat sich was geändert?
        neuZentrum = true;
        zp.setX(xsum);
        zp.setY(ysum);
      }
    }
    return neuZentrum;
  }
}